{% extends 'structures/base.html' %}
{% load i18n %}
{% load static %}
{% load humanize %}

{% block details %}

    <!-- Nav tabs -->
    <ul class="nav nav-tabs" role="tablist">
        <li role="presentation" class="nav-item">
            <button class="nav-link active" aria-controls="summary" role="tab" data-bs-toggle="tab" data-bs-toggle="#summary">
                {% trans "Summary" %}
            </button>
        </li>
    </ul>

    <div class="card card-default card-tabs border-0">
        <div class="card-body tab-content">
            <div role="tabpanel" class="tab-pane show active" id="summary">
                {% include "structures/partials/statistics/structure_summary.html" %}
            </div>
        </div>
    </div>

    <!-- Export to Javascript -->
    {{ data_export|json_script:"export-data" }}

{% endblock %}

{% block extra_javascript %}
    {% include 'bundles/datatables-js-bs5.html' %}
    {% include "bundles/filterdropdown-js.html" %}
    <script type="application/javascript" src="{% static 'structures/js/global.js' %}"></script>
    <script type="application/javascript" src="{% static 'structures/js/statistics.js' %}"></script>

{% endblock %}

{% block extra_css %}
    {% include 'bundles/datatables-css-bs5.html' %}
    <link rel="stylesheet" href="{% static 'structures/css/global.css' %}" type="text/css" media="screen">
    <link rel="stylesheet" href="{% static 'structures/css/main.css' %}" type="text/css" media="screen">
    <link rel="stylesheet" href="{% static 'structures/css/statistics.css' %}" type="text/css" media="screen">
{% endblock %}

{% block extra_script %}

{% endblock %}
