{% extends 'allianceauth/base-bs5.html' %}
{% load i18n %}
{% load navactive %}

{% block page_title %}{% translate "Structures" %}{% endblock %}

{% block header_nav_brand %}{% translate "Structures" %}{% endblock %}

{% block header_nav_collapse_left %}
    {% if perms.structures.view_corporation_structures or perms.structures.view_alliance_structures or perms.structures.view_all_structures %}
        <li class="nav-item">
            <a class="nav-link {% navactive request 'structures:structure_list' %}" href="{% url 'structures:structure_list' %}">
                {% translate 'Structure List' %}
            </a>
        </li>

        <li class="nav-item">
            <a class="nav-link {% navactive request 'structures:statistics' %}" href="{% url 'structures:statistics' %}">
                {% translate 'Statistics' %}
            </a>
        </li>
    {% endif %}

    <li class="nav-item me-auto">
        <a class="nav-link {% navactive request 'structures:public' %}" href="{% url 'structures:public' %}">
            {% translate 'Public' %}
        </a>
    </li>

{% endblock header_nav_collapse_left %}

{% block header_nav_collapse_right %}
    {% if perms.structures.add_structure_owner %}
        <li class="nav-item">
            <a class="btn btn-success me-2" href="{% url 'structures:add_structure_owner' %}" title="{% translate 'Add new structure owner or add new sync character to an existing structure owner' %}">
                <i class="fa-solid fa-plus"></i>&nbsp;Add owner
            </a>
        </li>
    {% endif %}
{% endblock header_nav_collapse_right %}

{% block header_nav_user_character_control %}
    {% comment %}
        Leave empty to remove user control nav items for this app
    {% endcomment %}
{% endblock %}

{% block content %}
    {% block details %}{% endblock %}
{% endblock %}
